import pandas as pd

def create_calendar_table(start, end, columns = ['day','month','year','week','weekday','month_name','weekday_name'],
                    locale=None):
    calendar = pd.DataFrame(pd.date_range(start=start, end=end),columns=['date'])
    if 'day' in columns:
        calendar['day'] = calendar.date.dt.day
    if 'month' in columns:
        calendar['month'] = calendar.date.dt.month
    if 'year' in columns:
        calendar['year'] = calendar.date.dt.year
    if 'week' in columns:
        calendar['week'] = calendar.date.dt.week
    if 'weekday' in columns:
        calendar['weekday'] = calendar.date.dt.weekday
    if 'month_name' in columns:
        calendar['month_name'] = calendar.date.dt.month_name()
    if 'weekday_name' in columns:
        calendar['weekday_name'] = calendar.date.dt.day_name()
    return calendar


import os, json

def open_json(json_file):
    with open(json_file) as data:
        return json.load(data)

def write_dotenv(env, file=".env"):
    """
    python-dotenv dont have any function to overwrite .env file
    write_dotenv is just a write variables in text file
    """
    with open(".env", "w") as f:
        for i in env:
            f.write(f'{i}={env[i]}\n')


## importing libs
import pandas as pd
import json
from sqlalchemy import create_engine
import psycopg2

def get_sql_table(query, engine, close_connection = False):
    """
    make a query in a database
    """
    ## open connection
    engine.connect()
    
    ## make select query
    table = pd.read_sql(query, engine)

    # ## close connection
    if close_connection:
        engine.close()
    return table

def write_sql_table(table, table_name, schema, engine, if_exists = 'append', chunksize = 10_000, index= False, close_connection = False):
    """
    if_exists => ["append","replace"]
    """

    ## open connection
    engine.connect()

    ## write or replace table
    table.to_sql(name = table_name,
                 if_exists = if_exists,
                 con = engine,
                 schema=schema,
                 method='multi',
                 chunksize = chunksize,
                 index = index)

    # ## close connection
    if close_connection:
        engine.close()


from jh_utils.data.pandas.sql import get_sql_table
from sqlalchemy import inspect

def get_top_rows(table, schema, engine, n=1):
    return get_sql_table(f'select * from {schema}.{table} dc limit {n}',engine)

def create_table_structure(pandas_df, table_name, engine, schema, index = False, if_exists = 'append'):
    pandas_df.to_sql(name = table_name,
                    con = engine, 
                    schema=schema, index=index, if_exists = if_exists)
    delete_table(table_name, schema, engine)

def get_schemas(engine):
    insp = inspect(engine)
    return insp.get_schema_names()

def create_schema(schema_name, engine):
    if not engine.dialect.has_schema(engine, schema_name):
        engine.execute(sa.schema.CreateSchema(schema_name))
    else:
        return 'Already exists'

def get_tables(schema, engine):
    insp = inspect(engine)
    return insp.get_table_names(schema)

def drop_table(table, schema, engine):
    conn = engine.connect()
    conn.execute(f'DROP TABLE IF EXISTS {schema}.{table};')
    conn.close()

def delete_table(table_name, schema, engine):
    conn = engine.connect()
    conn.execute(f'DELETE FROM {schema}.{table_name}')
    conn.close()

def drop_schema(schema, engine):
    conn = engine.connect()
    conn.execute(f'DROP SCHEMA {schema}')
    conn.close()


import pandas as pd

def crossjoin(column_1,column_2):
    """
    Create a crossjoin dataframe based on 2 columns
    """
    columns_name = [column_1.name,column_2.name]
    column_1 = pd.DataFrame(column_1.unique())
    column_2 = pd.DataFrame(column_2.unique())
    column_1['key'], column_2['key'] = 0,0
    df_ret = column_1.merge(column_2,on='key').drop(columns = 'key')
    df_ret.columns = columns_name
    return df_ret
    
##
def get_na_by_column(df):
    ret = df.isnull().mean()
    return ret

def clean_na(df, na_value = ''):
    ## Na from dataset

    df.index = df['date_time']
    df = df.drop(columns=['date_time'])

    df = df.replace(to_replace=na_value,value=np.NaN)
    df = df[df.iloc[:,1].first_valid_index():]
    df = df.fillna(method='ffill')
    return df

